#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# JLR InControl plugin
# Copyright (c) 2023 Alan Carter. All rights reserved.
#

################################################################################
# Imports
################################################################################

import jlrpy
import time


################################################################################
# Globals
################################################################################

def updateVar(name, value, folder=0):

    if name not in indigo.variables:
        indigo.variable.create(name, value=value, folder=folder)
    else:
        indigo.variable.updateValue(name, value)

################################################################################
class Plugin(indigo.PluginBase):

    ########################################
    # Class properties
    ########################################

    def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
     
        indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
        self.userName = pluginPrefs.get("userName", "")
        self.userPass = pluginPrefs.get("userPass", "")
        self.showBattInNotes = pluginPrefs.get("battNotes", False)
        self.logComms = pluginPrefs.get("logComms", False)
        self.lastRun = time.time()
    
    ########################################
    def __del__(self):
    
        indigo.PluginBase.__del__(self)
        
    ########################################
    def startup(self):
    
        indigo.server.log("Starting JLR plugin")
        self.createDevices()
        # Pause to allow gap between API calls
        self.sleep(5)
        self.getStatuses()


    def deviceStartComm(self, dev):
    
        dev.stateListOrDisplayStateIdChanged()
        return


    def shutdown(self):
    
        indigo.server.log("Stopping JLR plugin")
        pass
  
    ########################################
    
    def runConcurrentThread(self):
    
        indigo.server.log("Starting JLR monitoring thread")
        try:
        	while True:
        		self.sleep(100)
        		self.getStatuses()
        		self.sleep(500)
        		 
        except self.StopThread:
            self.shutdown()


    ########################################
    # Called when user prefs are changed
    
    def closedPrefsConfigUi(self, valuesDict, userCancelled):
    
        # If the user saved preferences, update logging parameters
        if userCancelled == False:

            battNotes = self.showBattInNotes
            self.showBattInNotes = valuesDict.get("battNotes", False)
            if self.showBattInNotes != battNotes:
                if self.showBattInNotes:
                    indigo.server.log("Charge % will be shown in Notes column")
                else:
                    indigo.server.log("Charge % will not be shown in Notes column")
                    
            logComms = self.logComms
            self.logComms = valuesDict.get("logComms", False)
            if self.logComms != logComms:
                if self.logComms:
                    indigo.server.log("Comms logging is on")
                else:
                    indigo.server.log("Comms logging is off")


    def createDevices(self):

        self.c = jlrpy.Connection(self.userName, self.userPass)
        for car in range (0, len(self.c.vehicles)):
            v = self.c.vehicles[car]
            health = v.get_health_status()
            if self.logComms:
            	indigo.server.log(str(health))
            vin = health['vehicleId']
            device = None
            for dev in indigo.devices.iter("self"):
                if dev.address == vin:
                    device = indigo.devices[dev.name]
            if device == None:
                indigo.server.log("Creating device for Vehicle %s" % (str(car)))
                nameC = "Vehicle (%s)" % (str(car))
                device = indigo.device.create(protocol=indigo.kProtocol.Plugin, address=vin, name=nameC, pluginId="com.racarter.indigoplugin.jlr", deviceTypeId="jlrVehicle", props={})


    def getStatuses(self):

        if (time.time() - self.lastRun) < 3:
            self.sleep(2)
        #c = jlrpy.Connection(self.userName, self.userPass)
        self.sleep(2)
        for car in range (0, len(self.c.vehicles)):
            v = self.c.vehicles[car]
            self.sleep(4)
            health = v.get_health_status()
            if self.logComms:
                indigo.server.log(str(health))
            vin = health['vehicleId']
            self.sleep(2)
            data = v.get_position()
            if self.logComms:
                indigo.server.log(str(data))
            pos = data['position']
            lat = pos['latitude']
            long = pos['longitude']
            if self.logComms:
                indigo.server.log(str(v.get_status()))
            for dev in indigo.devices.iter("self"):
                if dev.address == vin:
                    device = indigo.devices[dev.name]
            if device != None:
                key_value_list = [
                    {'key':'all_doors_locked', 'value':v.get_status('DOOR_IS_ALL_DOORS_LOCKED')},
                    {'key':'ev_charging_status', 'value':v.get_status('EV_CHARGING_STATUS')},
                    {'key':'ev_state_of_charge', 'value':v.get_status('EV_STATE_OF_CHARGE')},
                    {'key':'ev_minutes_to_fully_charged', 'value':v.get_status('EV_MINUTES_TO_FULLY_CHARGED')},
                    {'key':'ev_range_on_battery_miles', 'value':v.get_status('EV_RANGE_ON_BATTERY_MILES')},
                    {'key':'combined_range_miles', 'value':v.get_status('EV_PHEV_RANGE_COMBINED_MILES')},
                    {'key':'fuel_level_perc', 'value':v.get_status('FUEL_LEVEL_PERC')},
                    {'key':'tyre_pressure_front_left', 'value':round(int(v.get_status('TYRE_PRESSURE_FRONT_LEFT')) / 10)},
                    {'key':'tyre_pressure_front_right', 'value':round(int(v.get_status('TYRE_PRESSURE_FRONT_RIGHT')) / 10)},
                    {'key':'tyre_pressure_rear_left', 'value':round(int(v.get_status('TYRE_PRESSURE_REAR_LEFT')) / 10)},
                    {'key':'tyre_pressure_rear_right', 'value':round(int(v.get_status('TYRE_PRESSURE_REAR_RIGHT')) / 10)},
                    {'key':'latitude', 'value':lat},
                    {'key':'longitude', 'value':long},
                ]
                dev.updateStatesOnServer(key_value_list, clearErrorState=False)
                dev.updateStateImageOnServer(indigo.kStateImageSel.EnergyMeterOff)
                if self.showBattInNotes:
                    dev.description = "SOC "+v.get_status('EV_STATE_OF_CHARGE')+"%"
                    dev.replaceOnServer()
                else:
                    dev.description = ""
                    dev.replaceOnServer()    
        self.lastRun = time.time()
                    

    #######################################
    # Action functions
    #######################################

    def getStatus(self, pluginAction):
    
        self.getStatuses()
 
